<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Util;

class ProcessosController extends Crud {

    protected $dataSourceName = 'des_processo_busca';

    protected $parameterBagColumns = array('NRORG', 'DTATUAL', 'DTSOLICITA', 'DTLIMSOLICITA','P_NRORG' , 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, \HCM\BTL\Services\ProcessosService $processosService){
        $this->processosService = $processosService;
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function getProcessos($request, $response) {

        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            $fieldsUnset = array('SALARIOMINIMO', 'SALARIOMAXIMO', 'DTSOLICITA', 'DTLIMSOLICITA', 'FILTRO');
            $decode = array('IDPUBLICAVAGA');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    unset($filters[$key]);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            $result = $this->processosService->getProcessos($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getProcessos', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getProcessosAtivos($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
            }
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->processosService->getProcessosAtivos($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getProcessosAtivos', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getStatus($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $result = $this->processosService->getStatus($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getStatus', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getTemplates($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $result = $this->processosService->getTemplates($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTemplates', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getTipoprocessos($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $result = $this->processosService->getTipoprocessos($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTipoprocessos', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function adicionarUpdateProcesso($request, $response) {

        try {

            $dataset = $request->getDataSet();
            $dataset = $dataset->getRows();
            $dataset = $dataset[0];

            $result = $this->processosService->adicionarUpdateProcesso($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('save', array($result)));

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function deleteProcesso($request, $response) {

        try {
            
            $dataset = $request->getDataSet();
            $dataset = $dataset->getRows();
            $dataset = $dataset[0];

            $this->processosService->deleteProcesso($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getCandidatosParticipantes($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            $fieldsUnset = array('NRPROCESSO', 'IDADEMINIMA', 'IDADEMAXIMA', 'SALARIOMINIMO', 'SALARIOMAXIMO', 'P_NRORG', 'P_NRPROCESSO', 'CDIDIOM', 'DSINSTITUICAO', 'FILTRO');
            $fieldChange = array('NRGRAUINSTR', 'CDPAIS', 'SGESTADO', 'CDMUNICIPIO');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    unset($filters[$key]);
                }
                if(array_search($filter['columnName'], $fieldChange) !== false){
                    $filter = str_replace($filter['columnName'], 'GPE_PESSOAH.'.$filter['columnName'], $filter);
                    $filters[$key] = $filter;
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            $result = $this->processosService->getCandidatosParticipantes($parameters, $filters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getCandidatosParticipantes', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getProcessosCandidato($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $result = $this->processosService->getProcessosCandidato($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getProcessosCandidato', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function alterarIdPublicaVaga($request, $response) {

        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $result = $this->processosService->alterarIdPublicaVaga($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('alterarIdPublicaVaga', array($result)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
}