<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\FPA\ObrigacaoLegal\Dirf\Dirf;
use HCM\FPA\ObrigacaoLegal\Dirf\PercorreDirf;
use HCM\Geral\Controllers\RelatoriosHcmController;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciah;
use HCM\Util\Repositories;


class DIRFController extends Crud {

    protected $DIRFService;
    protected $dataSourceNameGet = 'getDIRF';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,EntityManager $entityManager,  RelatoriosHcmController $relatoriosHcmController){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->relatoriosHcmController = $relatoriosHcmController;
        parent::__construct($dataSourceManager);
    }

    public function getDIRF (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['NOMEFINAL']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['NOMEFINAL']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if($row['NOMEFINAL']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['NOMEFINAL']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
    }
    
    public function setDIRF(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_DIRF";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMDIRF', $row)) {
                    if ($row['NRFORMDIRF'] === '') {
                        $row['NRFORMDIRF'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMDIRF', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMDIRF'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMDIRF', 1, 12, true); 
                }
            }
            if(!array_key_exists('DSINICIO', $row)){
                $row['DSINICIO'] = null;
            }

            if(!array_key_exists('DSFIM', $row)){
                $row['DSFIM'] = null;
            }
            
            // define os campos de descrição de início e fim com base no critério de seleção passado pela row
            if($row['DSORDEM'] === "LISTA_VINCULO"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if(is_array($row['DSFIM'] && count($row['DSFIM']) > 0)) {
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                if(is_array($row['DSFIM'] && count($row['DSFIM']) > 0)) {
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            $row['NOMEINICIAL'] = $row['DSINICIO'];
            $row['NOMEFINAL'] = isset($row['DSFIM'])?$row['DSFIM']:null;
            $row['IDSOCIOSTENS'] = isset($row['IDSOCIOSTENS'])?$row['IDSOCIOSTENS']:"N";
            $row['IDDEPOJUDIC'] = isset($row['IDDEPOJUDIC'])?$row['IDDEPOJUDIC']:"N";
            $row['IDINSTINVEST'] = isset($row['IDINSTINVEST'])?$row['IDINSTINVEST']:"N";
            $row['IDPAGRENDEXT'] = isset($row['IDPAGRENDEXT'])?$row['IDPAGRENDEXT']:"N";
            $row['IDPAGPLANSAUD'] = isset($row['IDPAGPLANSAUD'])?$row['IDPAGPLANSAUD']:"N";
            $row['IDPAGRELCOPA'] = isset($row['IDPAGRELCOPA'])?$row['IDPAGRELCOPA']:"N";
            $row['IDPAGSITESPE'] = isset($row['IDPAGSITESPE'])?$row['IDPAGSITESPE']:"N";
            $row['IDRETVINCULOS'] = isset($row['IDRETVINCULOS'])?$row['IDRETVINCULOS']:"N"; 
            $row['IDRENDVINCULOS'] = isset($row['IDRENDVINCULOS'])?$row['IDRENDVINCULOS']:"N";
            $row['IDPROVINCULOS'] = isset($row['IDPROVINCULOS'])?$row['IDPROVINCULOS']:"N";
            
            if($row['IDTDVINCULOS'] == 'T'){
                $row['IDRETVINCULOS'] = "N"; 
                $row['IDRENDVINCULOS'] = "N";
                $row['IDPROVINCULOS'] = "N";
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function geraDirf(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        ini_set('memory_limit','2048M');
        error_reporting(E_ERROR | E_PARSE);
        $row = $request->getDataSet()->getRows()[0];
        try{
            $dirf = new Dirf($this->entityManager,$this->hcmParameters,$row['NRORG'],$row['NRFORMDIRF'],$this->hcmParameters->getDtCompetencia(),FPA::getNrModalidadeCalculo(),FPA::getOcorrenciaCalculo(), $this->relatoriosHcmController);
            $count = $dirf->getQdeRegsMagnet();
            if($count > 0)
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Já existe(m) $count registro(s) no magnético da DIRF para a Organização ".$row['NRORG']." na competência ".$this->hcmParameters->getDtCompetencia().".Se deseja gerar uma nova DIRF para esta competência por favor retroceda antes. A DIRF e o Magnético não foram gerados."));
            else{
                $res = $dirf->gerar();
                if($dirf->getMensagens()){
                    $mensagens = array();
                    $return = explode(Dirf::ql_msg, $dirf->getMensagens());
                    foreach($return as $key=>$msg){
                        $mensagens[$key] = array("MENSAGEM" => $msg);
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('mensagens', $mensagens));
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Não foi possível gerar a DIRF, Favor verificar as mensagens'));            
                }
                else{
                    if($res){
                        $return[] = array('NMARQUIVO' => $res);
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
                    }
                    $relatorio[] = array('NMREPORT' => $dirf->geraRelatorio());
                    $relatorio[] = array('NMREPORT' => $dirf->geraRelatorioAnalitico());
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('report', $relatorio));
                    $response->addNotification(new Notification('DIRF gerada com sucesso!','success'));
                }
            }
        } catch(\Exception $e){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage()));
        }
    }
    
    public function geraDirfOtimizada(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        ini_set('memory_limit','2048M');
        error_reporting(E_ERROR | E_PARSE);

        $dataSet = $request->getDataSet()->getRows()[0];
        
        try{
            $dataSet['TP_PROCESSAMENTO'] = 'COMPETENCIA_MENSAL';

            $percorreDirf = new PercorreDirf($this->entityManager, $this->hcmParameters, $dataSet);
            $resultado = $percorreDirf->percorre();

            if($resultado['ARQUIVO']){
                $row = array('NMARQUIVO' => FPA::getTempDirectory().DIRECTORY_SEPARATOR.$resultado['ARQUIVO']);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', [$row]));
                $message = new \Zeedhi\Framework\DTO\Response\Message(
                    'finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
            }
            if($resultado['RETROCESSO']) {
                 $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Retrocedido com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            }
            
            if ($resultado['ABORT'] === true) {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'SEFIP'));
            }elseif ($resultado['ERRO'] === false) {
                $filtroRelatorio = $percorreDirf->geraRelatorio();
                $caminhoRelatorio = $this->relatoriosHcmController->setRelatoriosHcmBackend($filtroRelatorio, 'informe_rendimentos', null, null);
                $relatorio[] = array('NMREPORT' => $caminhoRelatorio);
                
                $filtroRelatorioAnalitico = $percorreDirf->geraRelatorioAnalitico();
                $caminhoRelatorioAnalitico = $this->relatoriosHcmController->setRelatoriosHcmBackend($filtroRelatorioAnalitico, 'analitico_dirf', null, null);
                $relatorio[] = array('NMREPORT' => $caminhoRelatorioAnalitico);
                
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('report', $relatorio));
                $message = new \Zeedhi\Framework\DTO\Response\Message('Dirf finalizada com sucesso!', DTO\Response\Message::TYPE_MESSAGE);
            } else {
                $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'SEFIP'));
            }
            
            $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculo($resultado['CALCULO'], $this->hcmParameters->getNrorgTrabalho(), 'SEFIP');  
            
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            } else {
                $row = array();
                foreach ($message as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
        } catch(\Exception $e){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage()));
        }
    }
    
    public function retrocedeDirf(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        ini_set('memory_limit','2048M');
        error_reporting(E_ERROR | E_PARSE);
        $row = $request->getDataSet()->getRows()[0];
        try{
            $dirf = new Dirf($this->entityManager,$this->hcmParameters,$row['NRORG'],$row['NRFORMDIRF'],$this->hcmParameters->getDtCompetencia(),FPA::getNrModalidadeCalculo(),FPA::getOcorrenciaCalculo(), $this->relatoriosHcmController);
            $dirf->retroceder();
            if($dirf->getMensagens()){
                $mensagens = array();
                $return = explode(Dirf::ql_msg, $dirf->getMensagens());
                foreach($return as $key=>$msg){
                    $mensagens[$key] = array("MENSAGEM" => $msg);
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('mensagens', $mensagens));
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Não foi possível retroceder a DIRF, Favor verificar as mensagens'));            
            }else{
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));            
            }
        } catch(\Exception $e){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage()));
        }
    }
    
    public function deleteDIRF(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_DIRF";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function findValorReferencia(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dsReferencia     = "VR_DIRF_RENDIMENTOS";
            $nrorg            = $this->hcmParameters->getNrorgTrabalho();
            $dtcompetencia    = $this->hcmParameters->getDtCompetencia();

            $vrReferencia = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
                $nrorg, $nrorg, $dtcompetencia, $dsReferencia
            );

            if(is_null($vrReferencia)) {
                $vrReferencia = 0;
            }

            $response->addDataSet(new DataSet('referencia', array($vrReferencia)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    
}