<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1020Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS1020($parameters) {
        try {
            $sql = <<<SQL
/*Própria estrutura legal*/
SELECT EH.NRESTRUTURAM as NRCHAVE,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       EH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                     AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                     AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH H1
                                               WHERE H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                 AND H1.NRORG = EH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-1020'
              WHERE EC.NRORG = :P_NRORG
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = EH.NRESTRUTURAM
 WHERE EM.NRORG = :P_NRORG
   AND EM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
   AND (EM.DTFIMVIGENCIA IS NULL OR EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
   AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
  
UNION ALL
/*Tomadores abaixo da estrutura legal*/
SELECT DISTINCT EH.NRESTRUTURAM as NRCHAVE,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       EH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                     AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                     AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH H1
                                               WHERE H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                 AND H1.NRORG = EH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Estrutura Gerencial - Hierarquia Tomador/Gerencial*/
  JOIN HIERARQUIAORG HG ON HG.NRORG = EM.NRORG
                       AND HG.NRESTRUTURAMSUP = EM.NRESTRUTURAM
  JOIN TIPOHIERARQORG TG ON (TG.NRORG = HG.NRORG OR TG.NRORG = 0)
                        AND TG.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
                        AND TG.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                        AND TG.NRTIPOHIERARQORG = HG.NRTIPOHIERARQORG
  /*Empregador - Estrutura Legal - Hieraquia Legal/Gerencial*/
  JOIN HIERARQUIAORG HE ON HE.NRORG = HG.NRORG
                       AND HE.NRESTRUTURAMREL = HG.NRESTRUTURAMREL
                       AND HE.NRESTRUTURAMSUP = :P_NRESTRUTURAM
  JOIN TIPOHIERARQORG TE ON (TE.NRORG = EM.NRORG OR TE.NRORG = 0)
                        AND TE.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                        AND TE.NRTIPOESTRUTSUP = :P_NRTIPOESTRUTLEGAL
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-1020'
              WHERE EC.NRORG = :P_NRORG
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = EH.NRESTRUTURAM
 WHERE EM.NRORG = :P_NRORG
   AND EM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
   AND (EM.DTFIMVIGENCIA IS NULL OR EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
   AND EM.NRTIPOESTRUTURA = NVL(:P_NRTIPOESTRUTOMADOR, 7)
   AND (EM.NRESTRUTURAM = :P_NRESTRUTOMADOR OR :P_NRESTRUTOMADOR IS NULL)
   AND NOT EXISTS (SELECT 1
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO = 'S-1020'
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = :P_NRORG
                      AND EC.NRCHAVE = :P_NRESTRUTOMADOR
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'P' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                      AND EC.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 ORDER BY 1,4
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                "P_NRTIPOESTRUTGEREN" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
                "P_NRTIPOESTRUTOMADOR" => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
                "P_NRESTRUTURAM" => isset($parameters['NRESTRUTURAM']) ? $parameters['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($parameters['NRTPAMBIENTE']) ? $parameters['NRTPAMBIENTE'] : null,
                "P_NRESTRUTOMADOR" => isset($parameters['NRESTRUTTOMAD']) ? $parameters['NRESTRUTTOMAD'] : null,
                "P_CDCNPJEMPREGADOR" => isset($parameters['CDCNPJEMPREGADOR']) ? $parameters['CDCNPJEMPREGADOR'] : null
            );
           
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
     public function getDadosS1020($params){
         try {
            $dados = $this->getDadosPrincipais($params['NRENVIO'], $params['NRCHAVE'], $params['NRESTRUTURAM'], $params['NRTPAMBIENTE'], $params['DTMESCOMPETENC']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
     }

    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc) {
        $sql = <<<SQL
SELECT EM.NRESTRUTURAM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --infoLotacao
       --ideLotacao
       EM.NRESTRUTURAM codLotacao,
       TO_CHAR(GREATEST(EMP.DTINIVIGENCIA,EM.DTINIVIGENCIA,TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EM.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))),'YYYY-MM') iniValid,
       TO_CHAR(EM.DTFIMVIGENCIA, 'YYYY-MM') fimValid,
       --dadosLotacao
       TL.CDESOCIAL tpLotacao,
       CASE WHEN TL.CDESOCIAL IN ('01', '10', '21', '24', '90', '91') THEN '' ELSE DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') END dadosLotacao_tpInsc,
       CASE WHEN TL.CDESOCIAL IN ('01', '10', '21', '24', '90', '91') THEN '' ELSE PN.NRINSCRICAOPARC END dadosLotacao_nrInsc,
       --fpasLotacao
       EH.CDFPAS fpas,
       EH.CDTERCEIRO codTercs,
       '' codTercsSusp,
       --infoProcJudTerceiros
       --procJudTerceiro : CRIAR SELECT
       --infoEmprParcial
       DECODE(TL.CDESOCIAL,'02','','') tpInscContrat,
       DECODE(TL.CDESOCIAL,'02','','') nrInscContrat,
       DECODE(TL.CDESOCIAL,'02','','') tpInscProp,
       DECODE(TL.CDESOCIAL,'02','','') nrInscProp
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.NRORG = EM.NRORG
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                               FROM ESTRUTURAH H1
                                              WHERE H1.NRORG = EH.NRORG
                                                AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    /*AND EH.DTMESCOMPETENC = P_DTMESCOMPETENCIA*/
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = EH.NRPARCNEGOCIO
                     AND PN.NRORG = EH.NRORG
  /*Tipo Lotação*/
  JOIN TIPOESTRUTURA TE ON TE.NRTIPOESTRUTURA = EM.NRTIPOESTRUTURA
                       AND (TE.NRORG = EM.NRORG OR TE.NRORG = 0)
  LEFT JOIN ESO_TPLOTATRIBUT TL ON TL.NRTPLOTATRIBUT = TE.NRTPLOTATRIBUT
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRESTRUTURAM = :P_NRCHAVE
   AND (EM.NRESTRUTURAM = :P_NRESTRUTTOMAD OR :P_NRESTRUTTOMAD IS NULL)
   AND (EM.NRTIPOESTRUTURA = NVL(:P_NRTIPOESTRUTOMADOR, 7) OR EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidCadIniProdRest(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidCadIniProducao(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidCadIniNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam, 
            "P_NRTIPOESTRUTOMADOR" => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
            "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
            "P_NRESTRUTTOMAD" => isset($params['NRESTRUTTOMAD']) ? $params['NRESTRUTTOMAD'] : null,
            //"P_DTMESCOMPETENCIA" => $dtmescompetenc
        );
        return $teste =  $this->connection->fetchAssoc($sql, $params);
    }
    
}