<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\AlteEstabilidadeRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class AlteEstabilidadeService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, AlteEstabilidadeRepository $alteEstabilidadeRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->alteEstabilidadeRepository = $alteEstabilidadeRepository;
        $this->gpeAltestabvinc = Repositories::GPE_ALTESTABVINC;
    }

    public function nova($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc, $nrmotivoestab = null)
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeAltestabvinc $altestabvinc */
        $altestabvinc = new $this->gpeAltestabvinc;

        $altestabvinc->setNrorg($nrorg);
        $altestabvinc->setNrvinculom($nrvinculom);
        $altestabvinc->setDtiniestabvinc($dtiniestabvinc);
        $altestabvinc->setDtfimestabvinc($dtfimestabvinc);
        $altestabvinc->setNrmotivoestab($nrmotivoestab);

        $altestabvinc->setNrorginclusao($nrorg);
        $altestabvinc->setDtinclusao(DateUtil::getDataAtual());
        $altestabvinc->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $altestabvinc->setNrorgultatu($nrorg);
        $altestabvinc->setDtultatu(DateUtil::getDataAtual());
        $altestabvinc->setCdoperultatu($this->hcmParameters->getCdoperador());

        $altestabvinc->setNraltestabvinc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESTABVINC', 1, 12, true));

        $this->entityManager->persist($altestabvinc);
        $this->entityManager->flush($altestabvinc);
    }

    public function verificaPeriodoEstabilidadeVinculo($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc,$nraltestabvinc = null)
    {
        try {
            $estabvinc = $this->alteEstabilidadeRepository->retornaEstabilidadePorPeriodo($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc, $nraltestabvinc);
            if (!empty($estabvinc)) {
                $dtinicio = $estabvinc[0]->getDtiniestabvinc()->format('d/m/Y');
                if ($estabvinc[0]->getDtfimestabvinc()) {
                    $dtfim = $estabvinc[0]->getDtfimestabvinc()->format('d/m/Y');
                } else {
                    $dtfim = '';
                }
                throw new \Exception(Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESTABVINC_PERIDO_EXIST, array($dtinicio, $dtfim)), 10);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraEstabilidadeVinculo($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc)
    {
        try {
            $estabvinc = $this->alteEstabilidadeRepository->retornaEstabilidadePorPeriodo($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc);

            if (empty($estabvinc)) {
                $dtinicio = DateUtil::getDataDeString($dtiniestabvinc, DateUtil::FORMATO_BRASILEIRO, true);
                $dtfim = DateUtil::getDataDeString($dtfimestabvinc, DateUtil::FORMATO_BRASILEIRO, true);
                $this->nova($nrorg, $nrvinculom, $dtinicio, $dtfim);
                return Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESTABVINC_GERAR_ESTAB_SUCCESS, array($dtiniestabvinc, $dtfimestabvinc));
            } else {
                return Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESTABVINC_GERAR_ESTAB_ERROR, array($estabvinc[0]->getDtiniestabvinc(), $estabvinc[0]->getDtfimestabvinc()));
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

} 